package comunicacao;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketAddress;
import java.net.SocketException;
import java.util.Hashtable;

import userManagement.UserManager;

import comunicacao.data.MainBuffer;
import comunicacao.data.segment.Header;
import comunicacao.data.segment.HeaderFlags;
import comunicacao.data.segment.Segment;
import comunicacao.data.segment.SequenceNumber;
import comunicacao.exception.HeaderDontFitException;
import comunicacao.users.UserThread;

public class MasterThread extends Thread{
	
	private static DatagramSocket serverSocket;		
	private Hashtable<SocketAddress, UserThread> userThreads;
	private byte[] sendBuffer;
	private byte[] receiveBuffer;
	
	public MasterThread() throws SocketException{
		serverSocket = new DatagramSocket(1748);
		userThreads = new Hashtable<SocketAddress, UserThread>();
		
	}
	
	public void run(){
		
		try {
			DatagramPacket received = this.waitForIt();
			SocketAddress ipPort = received.getSocketAddress();
			if(this.userThreads.containsKey(ipPort)){
				UserThread userT = this.userThreads.get(ipPort);
				ObjectOutputStream out = new ObjectOutputStream(userT.getOutputStream());
				out.writeObject(received);
				
//				userT.setReceiveBuffer(received);
//				userT.notify();
			} else {
				Header header = new Header(received);
				if(header.getFlags().isSyn()){
					this.loginUser(received);
				}
				
			}
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (HeaderDontFitException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void loginUser(DatagramPacket syn) throws HeaderDontFitException, IOException{
		String message = "There is an user already logged with this email";
		
		if(!userThreads.containsKey(syn.getSocketAddress())){
			MainBuffer buffer = Server.requestBuffer(Header.HEADER_SIZE + Segment.MAXIMUM_DATA_SIZE);
			UserThread thread = new UserThread(syn, buffer);
			userThreads.put(syn.getSocketAddress(), thread);
			String emailSenha = new String(syn.getData(), Header.HEADER_SIZE, syn.getLength() - Header.HEADER_SIZE);
			int divisionIndex = emailSenha.indexOf("\n\n");
			UserManager.getInstace().login(emailSenha.substring(0, divisionIndex), emailSenha.substring(divisionIndex + 2), syn.getAddress(), syn.getPort());
			thread.start();			
			
		} else {
			DatagramPacket reset = new DatagramPacket(sendBuffer, Header.HEADER_SIZE + message.length(), syn.getAddress(), syn.getPort());
			Header header = new Header(new HeaderFlags(false, false, true, false, false, false, 0) , new SequenceNumber(0));
			header.putHeader(sendBuffer);
			
			byte[] messageByte = message.getBytes();
			for (int i = 0; i < messageByte.length; i++) {
				sendBuffer[Header.HEADER_SIZE + i] = messageByte[i];
			}
			
			serverSocket.send(reset);
			
		}
		
	}
	
	public static DatagramSocket getServerSocketInstance() throws SocketException{
		if(serverSocket == null){
			serverSocket = new DatagramSocket(1748);
		} 
		return serverSocket;
	}
	
	public DatagramPacket waitForIt() throws IOException{		
		DatagramPacket packet = new DatagramPacket(receiveBuffer,
				receiveBuffer.length);
		DatagramSocket serverSocket = Server.getServerSocket();
		serverSocket.receive(packet);		
		return packet;		
	}
	
	
}
