package comunicacao;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.util.LinkedList;
import java.util.Random;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import comunicacao.data.WindowSender;
import comunicacao.data.segment.Header;
import comunicacao.data.segment.HeaderFlags;
import comunicacao.data.segment.Segment;
import comunicacao.data.segment.SequenceNumber;
import comunicacao.data.segmentBuffer.SegmentBufferFile;
import comunicacao.data.segmentBuffer.SegmentBufferFillerThread;
import comunicacao.exception.HeaderDontFitException;
import comunicacao.sender.ReceiverThread;
import comunicacao.sender.SenderThread;
import comunicacao.util.UnsignedInteger;

public class Socket implements ISocket {
	private DatagramSocket socket;

	private int localPort;
	private InetAddress remoteAddress;
	private int remotePort;

	private byte[] receivingBuffer;
	private byte[] sendingBuffer;

	private boolean connected;

	private Random random;
	
	private ExecutorService executor;

	public Socket(InetAddress destinationIP, int destinationPort)
			throws SocketException {
		this.socket = new DatagramSocket();
		this.localPort = socket.getLocalPort();
		

		this.remoteAddress = destinationIP;
		this.remotePort = destinationPort;

		this.receivingBuffer = new byte[1024];
		this.sendingBuffer = new byte[1024];

		this.connected = false;

		this.random = new Random();
		
		this.executor = null;

	}

	public int getPort() {
		return this.socket.getLocalPort();
	}

	@Override
	public void sendObject(Serializable obj) throws IOException {
		// TODO Auto-generated method stub
		
		this.executor = Executors.newCachedThreadPool();
		ByteArrayOutputStream arrayOut = new ByteArrayOutputStream();
		ObjectOutputStream out = new ObjectOutputStream(arrayOut);
		out.writeObject(obj);
		byte[] bytes = arrayOut.toByteArray();
		out.close();
		DatagramPacket aux = new DatagramPacket(sendingBuffer,
				Header.HEADER_SIZE, remoteAddress, remotePort);
		Header header = new Header(new HeaderFlags(false, false, false, false,
				false, false, 0), new SequenceNumber(bytes.length));
		try {
			header.putHeader(sendingBuffer);
		} catch (HeaderDontFitException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		System.out.println(header.getSeqNum().value());
		socket.send(aux);

		SenderThread sender = new SenderThread(socket, bytes, bytes.length, this.remoteAddress, this.remotePort, executor);
		Thread thread = new Thread(sender);
		thread.start();
		try {
			thread.join();
			thread = null;
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		executor.shutdownNow();
		System.out.println("Sent object");
	}

	@Override
	public void sendFile(File file) throws IOException {
		// TODO Auto-generated method stub
		
		byte[] buffer = new byte[2000000];
		FileInputStream fileIn = new FileInputStream(file);
		long fileLength = file.length();
		long aux = 0;
		int read;
		String path = file.getAbsolutePath();
		String fileName = path.substring(path.lastIndexOf('\\') + 1);

		this.sendObject(fileLength);
		this.sendObject(fileName);
		
		this.executor = Executors.newCachedThreadPool();
		while (aux != fileLength) {
			read = fileIn.read(buffer);
			aux += read;

			SenderThread sender = new SenderThread(socket, buffer, read, this.remoteAddress, this.remotePort, executor);
			Thread thread = new Thread(sender);
			thread.start();
			try {
				thread.join();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}
		fileIn.close();
		executor.shutdownNow();

	}

	@Override
	public Serializable readObject() throws IOException, ClassNotFoundException {
		// TODO Auto-generated method stub
		DatagramPacket aux = new DatagramPacket(receivingBuffer,
				receivingBuffer.length);
		this.socket.receive(aux);
		Header header = new Header(aux);
		int size = (int) header.getSeqNum().value();
		System.out.println(size);

		byte[] receiveBuffer = new byte[size];
		ReceiverThread receiver = new ReceiverThread(socket, receiveBuffer,
				size, remoteAddress, remotePort);
		Thread thread = new Thread(receiver);
		thread.start();
		try {
			thread.join();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		ObjectInputStream in = new ObjectInputStream(new ByteArrayInputStream(
				receiveBuffer));
		Serializable ser = (Serializable) in.readObject();
		in.close();

		return ser;
	}

	@Override
	public File readFile() throws IOException, ClassNotFoundException {
		FileOutputStream fileOut = null;
		File file;
		long fileLength;
		String fileName;
		int aux = 0, read = 0;
		byte[] buffer = new byte[2000000];

		do {
			file = new File("temp\\" + random.nextInt(100) + ".yamtemp");
		} while (file.exists());

		try {
			fileOut = new FileOutputStream(file, true);
		} catch (FileNotFoundException e) {
			// Won't happen
			e.printStackTrace();
		}

		fileLength = (Long) this.readObject();

		fileName = (String) this.readObject();
		System.out.println(fileName);

		while (aux != fileLength) {
			if (fileLength - aux < buffer.length) {
				read = (int) (fileLength - aux);
			} else {
				read = buffer.length;
			}

			ReceiverThread receiver = new ReceiverThread(socket, buffer, read,
					remoteAddress, remotePort);
			Thread thread = new Thread(receiver);
			thread.start();
			try {
				thread.join();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			aux += read;
			System.out.println("Total: " + fileLength + " Received: " + aux);
			fileOut.write(buffer, 0, read);
		}

		fileOut.close();

		File definite = new File("temp\\" + fileName);
		file.renameTo(definite);

		return definite;
	}

	/**
	 * Method used for establishing a connection with a server.
	 * 
	 * @return true if the connection was successfully established, false
	 *         otherwise
	 */
	@Override
	public boolean connect() throws IOException {
		// TODO Auto-generated method stub
		DatagramPacket sendingPacket, receivePacket;
		Header sendingHeader, receiveHeader;

		/*------------------------------------SENDING SYN---------------------------------------*/
		sendingPacket = new DatagramPacket(sendingBuffer, Header.HEADER_SIZE,
				remoteAddress, remotePort);
		sendingHeader = new Header(new HeaderFlags(true, false, false, false,
				false, false, 0), new SequenceNumber(0));
		try {
			sendingHeader.putHeader(sendingBuffer);
		} catch (HeaderDontFitException e) {
			// Won't happen
			e.printStackTrace();
		}

		this.socket.send(sendingPacket);
		System.out.println("Socket sent a syn");

		/*---------------------------------WAITING FOR SYNACK-----------------------------------*/
		boolean receivedSynAck = false;

		this.socket.setSoTimeout(2000);
		receivePacket = new DatagramPacket(receivingBuffer,
				receivingBuffer.length);
		for (int i = 0; i < 5 && !receivedSynAck; i++) {
			try {
				System.out.println("Socket is waiting for synack");
				this.socket.receive(receivePacket);
				System.out.println("Socket has received a synack");
				receivedSynAck = true;
			} catch (SocketTimeoutException e) {
				this.socket.send(sendingPacket);
			}
		}

		receiveHeader = new Header(receivePacket);
		if (receiveHeader.getFlags().isSynack()) {
			/*--------------EXTRACT SERVER PORT NUMBER, SEND ACK-----------------------*/
			// extracting port number
			this.remotePort = (int) receiveHeader.getSeqNum().value();

			// send ack
			sendingHeader = new Header(new HeaderFlags(false, false, false,
					true, false, false, 0), new SequenceNumber(0));
			try {
				sendingHeader.putHeader(sendingBuffer);
			} catch (HeaderDontFitException e) {
				// Won't happen
				e.printStackTrace();
			}
			sendingPacket = new DatagramPacket(sendingBuffer, Header.HEADER_SIZE, remoteAddress, remotePort);
			this.socket.send(sendingPacket);
			System.out.println("Socket sent an ack");

//			this.socket.setSoTimeout(10000);
//
//			while (!this.connected) {
//				try {
//					this.socket.receive(receivePacket);
//					this.socket.send(sendingPacket);
//				} catch (SocketTimeoutException e) {
//					this.connected = true;
//				}
//			}
//			this.socket.setSoTimeout(0);
			
			this.connected = true;
			return true;

		}

		return false;
	}

	@Override
	public void close() throws IOException {
		/*--------------------------------SENDING FYN------------------------------------*/
		DatagramPacket fyn = new DatagramPacket(sendingBuffer,
				Header.HEADER_SIZE, remoteAddress, remotePort);

		Header sendingHeader = new Header(new HeaderFlags(false, true, false,
				false, false, false, 0), new SequenceNumber(0));
		try {
			sendingHeader.putHeader(sendingBuffer);
		} catch (HeaderDontFitException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		try {
			this.socket.send(fyn);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// TODO Auto-generated method stub

		/*------------------------------WAITING FYNACK---------------------------------*/
		boolean receivedFynAck = false;

		this.socket.setSoTimeout(2000);
		DatagramPacket fynAck = new DatagramPacket(receivingBuffer,
				receivingBuffer.length);
		for (int i = 0; i < 5 && !receivedFynAck; i++) {
			try {
				this.socket.receive(fynAck);
				receivedFynAck = true;
			} catch (SocketTimeoutException e) {
				this.socket.send(fyn);
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		Header receiveHeader = new Header(fynAck);
		if (receiveHeader.getFlags().isFyn()
				&& receiveHeader.getFlags().isAck()) {
			DatagramPacket ack = new DatagramPacket(sendingBuffer,
					Header.HEADER_SIZE, remoteAddress, remotePort);
			sendingHeader = new Header(new HeaderFlags(false, false, false,
					true, false, false, 0), new SequenceNumber(0));
			try {
				sendingHeader.putHeader(sendingBuffer);
			} catch (HeaderDontFitException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			this.socket.send(ack);

			boolean ackSent = false;

			while (!ackSent) {
				this.socket.setSoTimeout(30000);
				try {
					this.socket.receive(fynAck);
					this.socket.send(ack);
				} catch (SocketTimeoutException e) {
					this.socket.close();
					this.connected = false;
					ackSent = true;
				}
			}
		}

	}

	@Override
	public void sendFile(File file, Object fileState) {
		// TODO Auto-generated method stub

	}
	
	
	public DatagramSocket getDatagramSocket(){
		return this.socket;
	}

	@Override
	public InetAddress getRemoteAddress() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public int getRemotePort() {
		// TODO Auto-generated method stub
		return 0;
	}

}
