package comunicacao.data;

import java.util.concurrent.atomic.AtomicBoolean;

import comunicacao.data.segment.Header;
import comunicacao.data.segment.Segment;
import comunicacao.data.segment.SequenceNumber;
import comunicacao.data.segmentBuffer.SegmentBuffer;
import comunicacao.util.UnsignedInteger;

public class WindowSender {

	// Indexes in the segmentBuffer
	private int windowBase;
	

	//TODO take the congestion control to a different class?
	// Congestion control
	private int congWin;// Congestion window max size
	private int threshold;// Limit for slow start algorithm
	private int numberOfAckedSegments;// Counter for congestion avoidance
										// algorithm
	private int flightSize;// Amount of data currently sent and not acked.

	// Connection state variable
	// TODO this variable is probably going to the Socket class
	private AtomicBoolean connectionOpenned;

	public WindowSender( int windowBase) {
		super();

		this.windowBase = windowBase;
		

		this.congWin = 2;
		// TODO Can the threshold start as the maximum size of the window?
		this.threshold = 4000;
		this.numberOfAckedSegments = 0;
		this.flightSize = 0;

		// TODO this variable is probably going to the Socket class
		this.connectionOpenned = new AtomicBoolean(true);

	}

	public synchronized void ackReceived() {
		// ---------------CONGESTION CONTROL-----------------------

		// Slow start
		if ((this.congWin < this.threshold)) {
			this.congWin++;
		} // Congestion Avoidance
		else {
			numberOfAckedSegments++;
			System.out.println("Congestion avoidance: "
					+ this.numberOfAckedSegments);
			if (numberOfAckedSegments >= this.congWin) {
				this.congWin++;
				this.numberOfAckedSegments = 0;
			}
		}
	}

	public synchronized AtomicBoolean getConnectionOpenned() {
		return this.connectionOpenned;
	}

	public synchronized String toString() {
		return "Current window stats: WindowBase:" + this.windowBase
				+ " congWing:" + this.congWin
				+ " Threshold: " + threshold;
	}

	public synchronized boolean isInWindow(int index) {
		if ((index >= this.windowBase && (index < this.windowBase
				+ this.congWin))
		/*
		 * || (this.windowEnd < this.windowBase && (index > this.windowBase ||
		 * index < this.windowEnd))
		 */) {
			return true;
		} else {
			return false;
		}
	}

	public synchronized void timeoutOccurred() {
		this.threshold = Math.max(this.congWin / 2, 2);
		this.congWin = 1;

		// this.threshold = Math.max(this.flightSize / 2, 2);

		System.out.println("TIMEOUT: " + this.congWin / 2 + " " + toString());
	}

	public synchronized void slideWindow() {

		this.windowBase++;

		System.out.println(toString());
	}

	public synchronized void incrementFlightSize() {
		System.out.println("New flight size: " + this.flightSize);
		this.flightSize++;
	}

	public synchronized void decrementFlightSize() {
		System.out.println("New flight size: " + this.flightSize);
		this.flightSize--;
	}

	public synchronized int getFlightSize() {
		return flightSize;
	}
	
	public synchronized int getWindowBase(){
		return this.windowBase;
	}

}
