package comunicacao.data.segment;

import java.io.Serializable;
import java.net.DatagramPacket;

import comunicacao.exception.HeaderDontFitException;

/**
 * @author Joo Pedro
 *
 */
public class Header implements Serializable{
	private HeaderFlags flags;
	private SequenceNumber seqNum;
	public static final int HEADER_SIZE = 5;

	public HeaderFlags getFlags() {
		return flags;
	}

	public void setFlags(HeaderFlags flags) {
		this.flags = flags;
	}

	public SequenceNumber getSeqNum() {
		return seqNum;
	}

	public void setSeqNum(SequenceNumber seqNum) {
		this.seqNum = seqNum;
	}


	public Header(DatagramPacket packet) {
		if (packet != null) {
			byte[] data = packet.getData();
			this.flags = new HeaderFlags(data[0]);
			this.seqNum = new SequenceNumber(data[1], data[2], data[3], data[4]);
		} else {
			throw new NullPointerException();
		}
	}
	
	public Header(byte[] packetData) {
		if (packetData != null) {
			byte[] data = packetData;
			this.flags = new HeaderFlags(data[0]);
			this.seqNum = new SequenceNumber(data[1], data[2], data[3], data[4]);
		} else {
			throw new NullPointerException();
		}
	}


	public Header(HeaderFlags flags, SequenceNumber sequenceNumber) {
		this.flags = flags;
		this.seqNum = sequenceNumber;
	}

	public void putHeader(byte[] data) throws HeaderDontFitException {
		if (data != null && data.length > Header.HEADER_SIZE) {
			if (this.flags != null) {
				data[0] = this.flags.getFlag();
			} else {
				data[0] = 0;
			}

			if (this.seqNum != null) {
				data[1] = this.seqNum.getLeftByte();
				data[2] = this.seqNum.getMiddleLeftByte();
				data[3] = this.seqNum.getMiddleRightByte();
				data[4] = this.seqNum.getRightByte();
			} else {
				data[1] = 0;
				data[2] = 0;
				data[3] = 0;
				data[4] = 0;
			}
		} else {
			throw new HeaderDontFitException();
		}
	}

	public void putHeader(byte[] data, int offset) throws HeaderDontFitException {
		if (offset < 0 || data == null){
			throw new IllegalArgumentException("Either the data is null, or the offset is negative");
		}
		
		
		if (data.length > Header.HEADER_SIZE && data.length >= offset + Header.HEADER_SIZE) {
			if (this.flags != null) {
				data[offset] = this.flags.getFlag();
			} else {
				data[offset] = 0;
			}

			if (this.seqNum != null) {
				data[offset + 1] = this.seqNum.getLeftByte();
				data[offset + 2] = this.seqNum.getMiddleLeftByte();
				data[offset + 3] = this.seqNum.getMiddleRightByte();
				data[offset + 4] = this.seqNum.getRightByte();
			} else {
				data[offset + 1] = 0;
				data[offset + 2] = 0;
				data[offset + 3] = 0;
				data[offset + 4] = 0;
			}

		} else {			
			throw new HeaderDontFitException();
		}
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((flags == null) ? 0 : flags.hashCode());
		result = prime * result + ((seqNum == null) ? 0 : seqNum.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Header other = (Header) obj;
		if (flags == null) {
			if (other.flags != null)
				return false;
		} else if (!flags.equals(other.flags))
			return false;
		if (seqNum == null) {
			if (other.seqNum != null)
				return false;
		} else if (!seqNum.equals(other.seqNum))
			return false;
		return true;
	}
	
	

}
