package comunicacao.data.segment;

import java.io.Serializable;

/**
 * Header FLAGS:</br></br>
 * 
 * 0____1___2____3____4___5___7</br> |SYN|FYN|RST|ACK|STR|FID|</br> SYN -
 * connection establishment</br> FYN - connection closing</br> RST - error in
 * connection</br> ACK - this packet contains a acknowledgment</br> FID - This
 * packet contains a file frag belonging to the file with the specified FID</br>
 * STR - The data in this packet specifies where to safe the file to be
 * transmitted with the file descriptor</br>
 * 
 * @author Joo Pedro
 * 
 */

public class HeaderFlags implements Serializable{
	
	private byte flag;	
	
	public byte getFlag() {
		return flag;
	}

	public void setFlag(byte flag) {
		this.flag = flag;
	}
	
	public HeaderFlags(boolean syn, boolean fyn, boolean rst, boolean ack, boolean str, boolean file, int id){
		this.flag = header(syn, fyn, rst, ack, str, file, id);
	}	

	public HeaderFlags(byte flag) {
		super();
		this.flag = flag;
	}

	public static byte header(boolean syn, boolean fyn, boolean rst, boolean ack, boolean str, boolean file, int id){
		byte returned = 0;
		
		if(syn){
			returned = (byte) 0x80;
		} 
		
		if(fyn){
			returned = (byte) (returned |  0x40);
		} 
		
		if(rst){
			returned = (byte) (returned | 0x20);
		}
		
		if(ack){
			returned = (byte) (returned | 0x10);
		}
		
		if(str){
			returned = (byte) (returned | 0x8);
		}
		
		if(file){
			returned = (byte) (returned | 0x4);
		}
		
		switch(id){		
		case 1:
			returned = (byte) (returned | 0x1);
		case 2:
			returned = (byte) (returned | 0x2);
		case 3:
			returned = (byte) (returned | 0x3);
		}
		
		return returned;
	}
	
	public  boolean isSyn(){
		if((this.flag & (byte) 0x80) == (byte)0x80){
			return true;
		}
		return false;		
	}
	
	public  boolean isSynack(){
		if((this.flag & (byte) 0x90) == (byte)0x90){
			return true;			
		}
		return false;
	}
	
	public  boolean isFyn(){
		if((this.flag & (byte) 0x40) == (byte)0x40){
			return true;
		}
		return false;
	}
	
	public  boolean isRst(){
		if((this.flag & (byte) 0x20) == (byte)0x20){
			return true;
		}
		return false;
	}
	
	public  boolean isAck(){
		if((this.flag & (byte) 0x10) == (byte)0x10){
			return true;
		}
		return false;
	}
	
	public  boolean isStr(){
		if((this.flag & (byte) 0x8) == (byte)0x8){
			return true;
		}
		return false;
	}
	
	public  boolean isFile(){
		if((this.flag & (byte) 0x4) == (byte)0x4){
			return true;
		}
		return false;
	}
	
	public  byte idValue(){
		
		 if((this.flag & (byte) 0x1) == (byte)0x1){
			return 1;
		} else if ((this.flag & (byte) 0x2) == (byte)0x2){
			return 2;
		} else if((this.flag & (byte) 0x3) == (byte)0x3){
			return 3;
		} else {
			return 0;
		}
	}
	
	public static boolean isSyn(byte headerByte){
		if((headerByte & (byte) 0x80) == (byte)0x80){
			return true;
		}
		return false;		
	}
	
	public static boolean isSynack(byte headerByte){
		if((headerByte & (byte) 0x90) == (byte)0x90){
			return true;			
		}
		return false;
	}
	
	public static boolean isFyn(byte headerByte){
		if((headerByte & (byte) 0x40) == (byte)0x40){
			return true;
		}
		return false;
	}
	
	public static boolean isRst(byte headerByte){
		if((headerByte & (byte) 0x20) == (byte)0x20){
			return true;
		}
		return false;
	}
	
	public static boolean isAck(byte headerByte){
		if((headerByte & (byte) 0x10) == (byte)0x10){
			return true;
		}
		return false;
	}
	
	public static boolean isStr(byte headerByte){
		if((headerByte & (byte) 0x8) == (byte)0x8){
			return true;
		}
		return false;
	}
	
	public static boolean isFile(byte headerByte){
		if((headerByte & (byte) 0x4) == (byte)0x4){
			return true;
		}
		return false;
	}
	
	public static byte idValue(byte headerByte){
		
		 if((headerByte & (byte) 0x1) == (byte)0x1){
			return 1;
		} else if ((headerByte & (byte) 0x2) == (byte)0x2){
			return 2;
		} else if((headerByte & (byte) 0x3) == (byte)0x3){
			return 3;
		} else {
			return 0;
		}
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + flag;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		HeaderFlags other = (HeaderFlags) obj;
		if (flag != other.flag)
			return false;
		return true;
	}
	
	

}
