package comunicacao.data.segmentBuffer;

import java.net.InetAddress;
import java.util.Hashtable;

import comunicacao.data.segment.Segment;
import comunicacao.data.segment.SequenceNumber;

/**
 * @author Joo Pedro
 * 
 */
public abstract class SegmentBuffer implements Runnable {

	//
	private int transmitionID;

	// Buffers to which the file will be segmented
	private byte[] byteBuffer;
	private Segment[] segmentBuffer;
	private int segmentBufferIndex;
	private Hashtable<SequenceNumber, Integer> segmentMapping;
	private long dataLength;

	// Segments destination info
	private InetAddress destinationIP;
	private int destinationPort;

	// Internal class variables
	private long readBytes;
	private boolean finishedLoading;

	// Thread responsable for loading this buffer
	private Thread bufferLoaderThread;

	public SegmentBuffer(int transmitionID, byte[] byteBuffer,
			Segment[] segmentBuffer, long dataLength, InetAddress destinationIP,
			int destinationPort) {
		super();

		this.transmitionID = transmitionID;
		this.byteBuffer = byteBuffer;
		this.segmentBuffer = segmentBuffer;
		this.segmentBufferIndex = 0;
		// TODO ajeitar essa inicializacao da hashtable, receber uma no
		// construtor, ou criar uma grande o suficiente pra nao haver rehashing
		this.segmentMapping = new Hashtable<SequenceNumber, Integer>();
		this.dataLength = dataLength;

		this.destinationIP = destinationIP;
		this.destinationPort = destinationPort;

		this.readBytes = 0;
		this.finishedLoading = false;

		this.bufferLoaderThread = null;
	}

	public synchronized Segment get(int segmentsegmentBufferIndex) {
		return segmentBuffer[segmentsegmentBufferIndex];
	}

	public synchronized Segment get(SequenceNumber seqNum) {
		return this.segmentBuffer[this.segmentMapping.get(seqNum)];
	}

	public int numberOfSegments() {
		// TODO Is it always the last position of the array? If the file does
		// not fill the buffer completelly?
		return this.segmentBuffer.length;
	}

	public int indexOf(Segment segment) {
		Integer index = this.segmentMapping.get(segment.getSequenceNumber());
		if (index == null) {
			return -1;
		} else {
			return index;
		}
	}

	public byte[] getByteBuffer() {
		return byteBuffer;
	}

	public void setByteBuffer(byte[] byteBuffer) {
		this.byteBuffer = byteBuffer;
	}

	public Segment[] getSegmentBuffer() {
		return segmentBuffer;
	}

	public void setSegmentBuffer(Segment[] segmentBuffer) {
		this.segmentBuffer = segmentBuffer;
	}

	public int getSegmentBufferIndex() {
		return segmentBufferIndex;
	}

	public void setSegmentBufferIndex(int segmentBufferIndex) {
		this.segmentBufferIndex = segmentBufferIndex;
	}

	public Hashtable<SequenceNumber, Integer> getSegmentMapping() {
		return segmentMapping;
	}

	public void setSegmentMapping(
			Hashtable<SequenceNumber, Integer> segmentMapping) {
		this.segmentMapping = segmentMapping;
	}

	public InetAddress getDestinationIP() {
		return destinationIP;
	}

	public void setDestinationIP(InetAddress destinationIP) {
		this.destinationIP = destinationIP;
	}

	public int getDestinationPort() {
		return destinationPort;
	}

	public void setDestinationPort(int destinationPort) {
		this.destinationPort = destinationPort;
	}

	public long getReadBytes() {
		return readBytes;
	}

	public void setReadBytes(long l) {
		this.readBytes = l;
	}

	public boolean isFinishedLoading() {
		return finishedLoading;
	}

	public void setFinishedLoading(boolean finished) {
		this.finishedLoading = finished;
	}

	public Thread getBufferLoaderThread() {
		return bufferLoaderThread;
	}

	public void setBufferLoaderThread(Thread bufferLoaderThread) {
		this.bufferLoaderThread = bufferLoaderThread;
	}

	public long getDataLength() {
		return dataLength;
	}

	public void setDataLength(long dataLength) {
		this.dataLength = dataLength;
	}

	public int getTransmitionID() {
		return transmitionID;
	}

	public void setTransmitionID(int bufferID) {
		this.transmitionID = bufferID;
	}

	@Override
	public abstract void run();

}
