package comunicacao.data.segmentBuffer;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;

import comunicacao.data.segment.Header;
import comunicacao.data.segment.HeaderFlags;
import comunicacao.data.segment.Segment;
import comunicacao.data.segment.SequenceNumber;
import comunicacao.exception.HeaderDontFitException;

public class SegmentBufferFile extends SegmentBuffer {
	// File this segmentBuffer is segmenting
	private File file;
	// TODO
	private int numberOfFileParts;
	private int currentFilePart;
	private long ammountOfDataSegmented;

	private BufferedInputStream in;
	
	private boolean threadRunning;

	public SegmentBufferFile(File file, int skipInFile, int transmitionID,
			byte[] byteBuffer, Segment[] segmentBuffer,
			InetAddress destinationIP, int destinationPort) throws IOException {
		//TODO remove thread.start from superconstructor
		super(transmitionID, byteBuffer, segmentBuffer, file.length(),
				destinationIP, destinationPort);
		
		this.file = file;
		this.numberOfFileParts = (int) Math.ceil((double) file.length()
				/ byteBuffer.length);
		this.currentFilePart = 0;
		this.ammountOfDataSegmented = 0;
		this.in = new BufferedInputStream(new FileInputStream(file));
		in.skip(skipInFile);
		this.setBufferLoaderThread(new Thread(this));
		this.getBufferLoaderThread().start();
		
	}

	public synchronized void loadSegmentsFromFile() throws IOException,
			HeaderDontFitException {
		
		while (!this.isFinishedLoading()
				&& (this.getSegmentBuffer()[this.getSegmentBufferIndex()] == null || this
						.getSegmentBuffer()[this.getSegmentBufferIndex()]
						.isAcked())) {
			
			if (this.getSegmentBuffer()[this.getSegmentBufferIndex()] != null){
				this.getSegmentMapping().remove(this.getSegmentBuffer()[this.getSegmentBufferIndex()].getSequenceNumber());
			}

			Header header = new Header(new HeaderFlags(false, false, false,
					false, false, true, this.getTransmitionID()),
					new SequenceNumber(this.ammountOfDataSegmented));
			header.putHeader(this.getByteBuffer(), (int) this.getReadBytes());

			this.setReadBytes(this.getReadBytes() + Header.HEADER_SIZE);

			int currentReadBytes = in.read(this.getByteBuffer(),
					(int) this.getReadBytes(), Segment.MAXIMUM_DATA_SIZE);
//			System.out.print("Loaded from file starting at bytes "
//					+ this.ammountOfDataSegmented + " ending at byte "
//					+ (this.ammountOfDataSegmented + currentReadBytes) + "\n");
			
			
			
//			System.out.print("Loaded data starting at byte "
//					+ this.getReadBytes() + "\n");
			this.getSegmentBuffer()[this.getSegmentBufferIndex()] = new Segment(
					new DatagramPacket(this.getByteBuffer(), (int) (this
							.getReadBytes() - Header.HEADER_SIZE),
							Header.HEADER_SIZE + currentReadBytes, this
									.getDestinationIP(), this
									.getDestinationPort()),
					header.getSeqNum());
//			System.out.println("Built segment at index "
//					+ this.getSegmentBufferIndex());
			this.getSegmentMapping().put(header.getSeqNum(),
					this.getSegmentBufferIndex());
			this.ammountOfDataSegmented += currentReadBytes;

			if (this.ammountOfDataSegmented == file.length()) {
				this.setFinishedLoading(true);
			}
			this.setReadBytes(this.getReadBytes() + currentReadBytes);

			if (this.getReadBytes() + Header.HEADER_SIZE
					+ Segment.MAXIMUM_DATA_SIZE >= this.getByteBuffer().length) {
				this.setReadBytes(0);
			}

			this.setSegmentBufferIndex(this.getSegmentBufferIndex() + 1);
			if (this.getSegmentBufferIndex() == this.getSegmentBuffer().length) {
				this.setSegmentBufferIndex(0);
				// TODO Is this really the place to increment it?
				this.currentFilePart++;
			}
			synchronized (this) {
				this.notify();
			}

		}

	}

	@Override
	public void run() {
		try {
			while (!this.isFinishedLoading()) {
				try {
					this.loadSegmentsFromFile();
					if (!this.isFinishedLoading()) {
						synchronized (this) {
							this.wait();
						}
					}
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (HeaderDontFitException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		} finally {
			try {
				in.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}

}
