package comunicacao.data.segmentBuffer;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.net.DatagramPacket;
import java.net.InetAddress;

import comunicacao.data.segment.Header;
import comunicacao.data.segment.HeaderFlags;
import comunicacao.data.segment.Segment;
import comunicacao.data.segment.SequenceNumber;
import comunicacao.exception.HeaderDontFitException;

public class SegmentBufferObject<T extends Serializable> extends SegmentBuffer {

	private T object;

	public SegmentBufferObject(T object, Segment[] segmentBuffer, byte[] byteBuffer,
			int transmitionID, InetAddress destinationIP, int destinationPort)
			throws IOException, HeaderDontFitException {
		super(transmitionID, byteBuffer, segmentBuffer, 0, destinationIP,
				destinationPort);
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		ObjectOutputStream out2 = new ObjectOutputStream(out);
		out2.writeObject(object);
		byte[] bytes = out.toByteArray();
		this.setDataLength(bytes.length);
		out2.close();

		ByteArrayInputStream in = new ByteArrayInputStream(bytes);

		int numberOfSegments = (int) Math.ceil((double) bytes.length
				/ Segment.MAXIMUM_DATA_SIZE);
		int currentSeqNumber = 0;
		int offset = 0;
		int readBytes;

		for (int i = 0; i < numberOfSegments; i++) {
			Header header = new Header(new HeaderFlags(false, false, false,
					false, true, false, transmitionID), new SequenceNumber(
					currentSeqNumber));
			header.putHeader(this.getByteBuffer(), offset);

			offset += Header.HEADER_SIZE;

			readBytes = in.read(getByteBuffer(), offset,
					Segment.MAXIMUM_DATA_SIZE);

			DatagramPacket packet = new DatagramPacket(getByteBuffer(), offset
					- Header.HEADER_SIZE, readBytes + Header.HEADER_SIZE,
					getDestinationIP(), getDestinationPort());
			offset += readBytes;
			currentSeqNumber += readBytes;

			this.getSegmentBuffer()[i] = new Segment(packet, header.getSeqNum());
		}

		this.object = object;
	}

	@Override
	public void run() {
		// TODO Auto-generated method stub

	}

}
