package comunicacao.packets;

import java.io.IOException;
import java.net.DatagramSocket;
import java.util.Hashtable;
import java.util.concurrent.Future;

import comunicacao.Server;
import comunicacao.data.WindowSender;
import comunicacao.data.segment.Segment;
import comunicacao.data.segment.SequenceNumber;
import comunicacao.exception.ClientDisconnectedException;
import comunicacao.sender.SenderThread;
import comunicacao.sender.TimeoutAlgorith;

public class SendingThread implements Runnable {

	private Segment segment;
	private int index;

	private long RTT;
	private TimeoutAlgorith transmissionTimeout;
	private long myTimeout;
	private int ammountOfTries;
	private boolean retransmited;

	private DatagramSocket socket;

	private boolean alive;
	
	private SenderThread parent;

	private Hashtable<SequenceNumber, SendingThread> threadMapping;

	public SendingThread(Segment segment, int index, TimeoutAlgorith transmissionTimeout,
			DatagramSocket socket,
			Hashtable<SequenceNumber, SendingThread> threadMapping2) {
		this.segment = segment;
		this.index = index;
		
		this.RTT = 0;
		this.transmissionTimeout = transmissionTimeout;
		this.myTimeout = this.transmissionTimeout.getTimeout();
		this.ammountOfTries = 0;

		this.alive = true;

		this.socket = socket;

		this.threadMapping = threadMapping2;

	}

	public synchronized boolean isAlive() {
		return this.alive;
	}

	public synchronized void setAlive(boolean alive) {
		this.alive = alive;
	}

	public void run() {
		System.out.println("Sending thread " + this.segment.getSequenceNumber().value() + " initialized");

		try {
			while (this.isAlive()) {
				System.out.println("Sending thread " + this.segment.getSequenceNumber().value() + " sending segment");
				this.socket.send(segment.getPacket());

				try {
					this.myTimeout = Math.max(this.transmissionTimeout.getTimeout(), 50);
					
					Thread.sleep(myTimeout);
					synchronized (this.segment) {
						if (segment.isAcked()) {
							System.out.println("Sending thread received a ack");
							this.setAlive(false);
						} else {
							this.retransmited = true;
							this.parent.timeoutOccurred();
							System.out.println("Sending thread " + this.segment.getSequenceNumber().value() + " timedout");
							if (!this.parent.getWindow().isInWindow(index)){
								this.setAlive(false);
							}
						}
					}
					

				} catch (InterruptedException e) {
					this.setAlive(false);
				}

			}

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println("Sending thread " + this.segment.getSequenceNumber().value() + " is dieing");
		this.threadMapping.remove(this.segment.getSequenceNumber());
	}

	

//	public boolean waitingPacket() throws ClientDisconnectedException {
//		try {
//
//			if (!retransmited) {
//
//				this.waitingForAck = true;
//				this.myTimeout = this.window.getTimeout();
//			} else {
//				if (this.myTimeout < 64000) {
//					this.myTimeout = this.myTimeout << 1;
//				} else {
//					this.ammountOfTries++;
//				}
//			}
//			// TODO Ask the monitor, what if the client falls down?
//			if (this.ammountOfTries == 5) {
//				throw new ClientDisconnectedException();
//			}
//			myTimeout = Math.max(myTimeout, 50);
//			System.out.println("Thread "
//					+ this.segment.getSequenceNumber().value()
//					+ " is going to sleep for " + this.myTimeout);
//			Thread.sleep(myTimeout);
//			// Timeout occurred:
//			if (!this.isInterrupted() && this.getWaitingForAck()) {
//				this.window.timeoutOccurred();
//				System.out.println("Thread "
//						+ segment.getSequenceNumber().value() + " timed out");
//				this.retransmited = true;
//				return false;
//			} else {
//				System.out.println("Thread "
//						+ this.segment.getSequenceNumber().value()
//						+ " was successfully interrupted");
//				return true;
//			}
//		} catch (InterruptedException e) {
//			System.out.println("Thread "
//					+ this.segment.getSequenceNumber().value()
//					+ " was successfully interrupted");
//			return true;
//		}
//	}

	public synchronized void recalculateTimeout() {
		if (!this.retransmited) {
			// this.window.decrementFlightSize();
			this.RTT = System.currentTimeMillis() - this.RTT;
			this.transmissionTimeout.calculateTimeout(RTT);
		}
	}

	public Segment getSegment() {
		return segment;
	}

}
